open Datatypes
open List0

open NatTools
open BsmlNat

open Bsml
open Stdlib.Base

(** *Data set generator   *)

(** Generating  data set ('a list)
    f : data generator for the position given as parameter
    local_size : size of the list of data to be generated for this processor
    proc_num : current processor number
val coq_local_float_list_generator :
  (int -> 'a) -> int -> int -> 'a List0.list
val local_float_list_generator :
  (int -> 'a) -> int -> int -> 'a List.list

*)

let coq_nil = []
let coq_cons (h,t) = h::t

let coq_local_simpl_list_generator f local_size proc_num=
  let rec data_aux i l =
    if i = 0 
    then l 
    else 
      let pos= i-1+proc_num*local_size in
	data_aux  (i -1)
	  (coq_cons ((f pos), l))
  in
    data_aux local_size coq_nil 

let local_simpl_list_generator f local_size proc_num=
  let rec data_aux i l =
    if i = 0 
    then l 
    else 
      let pos= i-1+proc_num*local_size in
	data_aux  (i -1) (((f pos))::l)
  in
    data_aux local_size []

(** Generating  data set : (float,'a) list
    f : data generator for the position given as parameter
    local_size : size of the list of data to be generated for this processor
    proc_num : current processor number
val coq_local_pos_data_list_generator :
  (int -> 'a) -> int -> int -> (float, 'a) Datatypes.prod List0.list
val local_float_list_generator :
  (int -> 'a) -> int -> int -> (float * 'a)  list

*)
let coq_local_pos_data_list_generator f local_size proc_num=
  let rec data_aux i l =
    if i = 0 
    then l 
    else 
      let pos= i-1+proc_num*local_size in
	data_aux  (i -1)
	(coq_cons ((Coq_pair  
  	  	      ((float_of_int pos) 
  	  		 ,  
  	  	       (f pos) 
  	  	      )), 
  	  	   l))  
  in
    data_aux local_size coq_nil 

let local_pos_data_list_generator f local_size proc_num=
  let rec data_aux i l =
    if i = 0 
    then l 
    else 
      let pos= i-1+proc_num*local_size in
	data_aux  (i -1) ((float_of_int pos, f pos)::l)
  in
    data_aux local_size []

(** Generating  random  data set *)
let random_data  local_data_generator max_value (local_size:int) (proc_num:int)  =
  let _ = Random.self_init() in
    local_data_generator (fun _ -> float_of_int (Random.int max_value - (max_value/2))) local_size proc_num
  
(** Generating  Fixed  data set *)
let fixed_data  local_data_generator local_size proc_num  =
  local_data_generator (fun pos ->  float_of_int (abs( pos-2*(pos mod (bsp_p*local_size/2)   )))) local_size proc_num


(** Balanced distributed Random data  *)
let balanced_random_data_par local_data_generator max_value  local_size =      
  mkpar (random_data   local_data_generator  max_value local_size)

(* (\** Balanced distributed Random data  *\) *)
(* let coq_balanced_random_data_par max_value  local_size = *)
(*   mkpar (random_data  coq_local_data_generator max_value local_size) *)


(* (\** Balanced distributed fixed data  *\) *)
(* let balanced_fixed_data_par local_size = *)
(*   mkpar (fixed_data  local_data_generator local_size) *)

(* (\** Balanced distributed fixed data  *\) *)
(* let coq_balanced_fixed_data_par local_size = *)
(*   mkpar (fixed_data coq_local_data_generator  local_size) *)
